(ns com.dreamsoft.calculator.token
  "Token related constants and functions to deal with them")

;;; Module local constants
(def dot
  "The valid dot token"
  ".")

(def figures
  "A vector of all valid number figure tokens"
  ["0" "1" "2" "3" "4" "5" "6" "7" "8" "9"])

(def plus "+")
(def minus "-")
(def times "×")
(def divided "÷")

(def operators
  "A vector of all valid operator tokens"
  [plus minus times divided])

(def ^:private open-parenthesis
  "The valid open-parenthesis token"
  "(")

(def ^:private close-parenthesis
  "The valid close-parenthesis token"
  ")")

(def ^:private children-needed
  {plus 2
   minus 2
   times 2
   divided 2
   open-parenthesis 1})

;;; Token related functions
(defn digit?
  "Check whether a token is a digit"
  [token]
  {:pre [(string? token)]}
  (contains? (set figures) token))

#_(defn- parse-int
  [token]
	{:pre [(every? digit? token)]}
  (js/Number.parseInt token))

#_(defn- parse-float
  [token]
	{:pre [(every? digit? token)]}
  (js/Number.parseFloat token))

; function "only-digits?" predeclaration for "append-digits" to keep them separate
(declare only-digits?)

(defn- append-digits
  "Create a new digits token with a figure appended"
  [digits token]
	{:pre [(digit? token) (only-digits? digits)]}
  [token digits])

(defn close-parenthesis?
  "Check whether a token is a close-parenthesis"
  [token]
	{:pre [(string? token)]}
  (= token close-parenthesis))

(defn dot?
  "Check whether a token is a dot"
  [token]
	{:pre [(string? token)]}
  (= token dot))

(defn operator?
  "Check whether a token is an operator"
  [token]
	{:pre [(string? token)]}
  (contains? operators token))

(defn open-parenthesis?
  "Check whether a token is an open-parenthesis"
  [token]
	{:pre [(string? token)]}
  (= token open-parenthesis))

;; not better place to put it in, should be in "com.dreamsoft.calculator.ast"

(defn only-digits? ; a "number" example: ["2" ["." ["1"]]] -> 1.2
	"Check whether or not a subtree is only made of dots and digits"
	[node]
	{:pre [(vector? node)]} ; using "ast/zipper?" instead ?
	(loop [[x & xs] node]
      (cond
       ((some-fn digit? dot?) x)
        (recur xs)
       (empty? xs)
        true
       :else
        false)))